import { AppRegistry } from 'react-native';
import BarButtonEventHandler from './handler/BarButtonEventHandler';
import LayoutCommandHandler from './handler/LayoutCommandHandler';
import DispatchCommandHandler from './handler/DispatchCommandHandler';
import ResultEventHandler from './handler/ResultEventHandler';
import VisibilityEventHandler from './handler/VisibilityEventHandler';
import Event from './Event';
import GardenModule from './GardenModule';
import NavigationModule from './NavigationModule';
export const { RESULT_CANCEL, RESULT_OK, RESULT_BLOCK } = NavigationModule.getConstants();
class NavigationImpl {
    dispatchHandler = new DispatchCommandHandler();
    resultHandler = new ResultEventHandler();
    buttonHandler = new BarButtonEventHandler();
    layoutHandler = new LayoutCommandHandler();
    visibilityHandler = new VisibilityEventHandler();
    constructor() {
        this.handleTabSwitch();
        this.layoutHandler.handleRootLayoutChange();
        this.visibilityHandler.handleComponentVisibility();
        this.resultHandler.handleComponentResult();
    }
    wrap;
    hoc;
    registerEnded = false;
    startRegisterComponent(hoc) {
        this.hoc = hoc;
        this.registerEnded = false;
        NavigationModule.startRegisterReactComponent();
    }
    endRegisterComponent() {
        if (this.registerEnded) {
            console.warn("Please don't call ReactRegistry#endRegisterComponent multiple times.");
            return;
        }
        this.registerEnded = true;
        NavigationModule.endRegisterReactComponent();
    }
    registerComponent(appKey, getComponentFunc, routeConfig) {
        if (routeConfig) {
            this.registerRoute(appKey, routeConfig);
        }
        let WrappedComponent = getComponentFunc();
        if (this.hoc) {
            WrappedComponent = this.hoc(WrappedComponent);
        }
        // build static options
        let options = this.bindBarButtonClickEvent('permanent', WrappedComponent.navigationItem) ||
            {};
        NavigationModule.registerReactComponent(appKey, options);
        let RootComponent = this.wrap(appKey)(WrappedComponent);
        AppRegistry.registerComponent(appKey, () => RootComponent);
    }
    _routeConfigs = new Map();
    registerRoute(moduleName, route) {
        this._routeConfigs.set(moduleName, route);
    }
    routeConfigs() {
        return this._routeConfigs;
    }
    setNavigationComponentWrap(wrap) {
        this.wrap = wrap;
    }
    addGlobalVisibilityEventListener(listener) {
        this.visibilityHandler.addGlobalVisibilityEventListener(listener);
        return {
            remove: () => this.visibilityHandler.removeGlobalVisibilityEventListener(listener),
        };
    }
    addVisibilityEventListener(sceneId, listener) {
        this.visibilityHandler.addVisibilityEventListener(sceneId, listener);
        return {
            remove: () => this.visibilityHandler.removeVisibilityEventListener(sceneId, listener),
        };
    }
    dispatch(sceneId, action, params = {}) {
        return this.dispatchHandler.dispatch(sceneId, action, params);
    }
    handleTabSwitch() {
        Event.listenTabSwitch((sceneId, from, to) => {
            this.dispatch(sceneId, 'switchTab', { from, to });
        });
    }
    setInterceptor(interceptor) {
        this.dispatchHandler.setInterceptor(interceptor);
    }
    setResult = (sceneId, resultCode, data = null) => {
        if (resultCode < RESULT_OK) {
            console.warn('`resultCode` can only be `RESULT_OK`, `RESULT_CANCEL` or an integer greater than 0.');
        }
        NavigationModule.setResult(sceneId, resultCode, data);
    };
    result(sceneId, resultCode) {
        return this.resultHandler.waitResult(sceneId, resultCode);
    }
    unmount(sceneId) {
        this.resultHandler.invalidateResultEventListener(sceneId);
        this.buttonHandler.unbindBarButtonClickEvent(sceneId);
    }
    setRoot(layout, sticky = false) {
        return this.layoutHandler.setRoot(layout, sticky);
    }
    setRootLayoutUpdateListener(willSetRoot = () => { }, didSetRoot = () => { }) {
        this.layoutHandler.setRootLayoutUpdateListener(willSetRoot, didSetRoot);
    }
    currentTab(sceneId) {
        return NavigationModule.currentTab(sceneId);
    }
    findSceneIdByModuleName(moduleName) {
        return NavigationModule.findSceneIdByModuleName(moduleName);
    }
    isStackRoot(sceneId) {
        return NavigationModule.isStackRoot(sceneId);
    }
    signalFirstRenderComplete(sceneId) {
        NavigationModule.signalFirstRenderComplete(sceneId);
    }
    currentRoute() {
        return NavigationModule.currentRoute();
    }
    routeGraph() {
        return NavigationModule.routeGraph();
    }
    setDefaultOptions(options) {
        GardenModule.setStyle(options);
    }
    setLeftBarButtonItem(sceneId, buttonItem) {
        const options = this.bindBarButtonClickEvent(sceneId, buttonItem);
        GardenModule.setLeftBarButtonItem(sceneId, options);
    }
    setRightBarButtonItem(sceneId, buttonItem) {
        const options = this.bindBarButtonClickEvent(sceneId, buttonItem);
        GardenModule.setRightBarButtonItem(sceneId, options);
    }
    setLeftBarButtonItems(sceneId, buttonItems) {
        const options = this.bindBarButtonClickEvent(sceneId, buttonItems);
        GardenModule.setLeftBarButtonItems(sceneId, options);
    }
    setRightBarButtonItems(sceneId, buttonItems) {
        const options = this.bindBarButtonClickEvent(sceneId, buttonItems);
        GardenModule.setRightBarButtonItems(sceneId, options);
    }
    setTitleItem(sceneId, titleItem) {
        GardenModule.setTitleItem(sceneId, titleItem);
    }
    updateOptions(sceneId, options) {
        GardenModule.updateOptions(sceneId, options);
    }
    updateTabBar(sceneId, options) {
        GardenModule.updateTabBar(sceneId, options);
    }
    setTabItem(sceneId, item) {
        if (!Array.isArray(item)) {
            item = [item];
        }
        GardenModule.setTabItem(sceneId, item);
    }
    setMenuInteractive(sceneId, enabled) {
        GardenModule.setMenuInteractive(sceneId, enabled);
    }
    setBarButtonClickEventListener(listener) {
        this.buttonHandler.setBarButtonClickEventListener(listener);
    }
    bindBarButtonClickEvent(sceneId, item) {
        return this.buttonHandler.bindBarButtonClickEvent(sceneId, item);
    }
}
export default new NavigationImpl();
