import { pathToRegexp, match } from 'path-to-regexp';
import { StackRouteHandler } from './stack';
import { ScreenRouteHandler } from './screen';
import { TabsRouteHandler } from './tabs';
import { DrawerRouteHandler } from './drawer';
import Navigation from '../Navigation';
function pathParams(path) {
    const pathMatch = match(path, {
        encode: encodeURI,
        decode: decodeURIComponent,
    })(path);
    return pathMatch ? pathMatch.params : {};
}
function queryParams(queryString) {
    return (queryString || '').split('&').reduce((result, item) => {
        if (item !== '') {
            const nextResult = result || {};
            const [key, value] = item.split('=');
            nextResult[key] = value;
            return nextResult;
        }
        return result;
    }, {});
}
class Router {
    constructor() {
        this.registerRouteHandler('screen', new ScreenRouteHandler());
        this.registerRouteHandler('stack', new StackRouteHandler());
        this.registerRouteHandler('tabs', new TabsRouteHandler());
        this.registerRouteHandler('drawer', new DrawerRouteHandler());
    }
    handlers = new Map();
    registerRouteHandler(layout, handler) {
        this.handlers.set(layout, handler);
    }
    convertConfigToData(moduleName, route) {
        let { path, dependency, mode } = route;
        let regexp;
        if (path) {
            if (!path.startsWith('/')) {
                route.path = '/' + route.path;
            }
            regexp = pathToRegexp(path);
        }
        mode = mode || 'push';
        return { moduleName, mode, path, regexp, dependency };
    }
    isRouteConfigInflated = false;
    routeDataSet = new Map();
    inflateRouteData() {
        if (this.isRouteConfigInflated) {
            return;
        }
        this.isRouteConfigInflated = true;
        const routeConfigs = Navigation.routeConfigs();
        for (const [moduleName, config] of routeConfigs) {
            const data = this.convertConfigToData(moduleName, config);
            this.routeDataSet.set(moduleName, data);
        }
    }
    interceptors = [];
    addInterceptor(interceptor) {
        this.interceptors.push(interceptor);
    }
    removeInterceptor(interceptor) {
        this.interceptors = this.interceptors.filter(item => item !== interceptor);
    }
    async intercept(path) {
        for (let interceptor of this.interceptors.values()) {
            const result = interceptor(path);
            const intercepted = await this.resolveInterceptResult(result);
            if (intercepted) {
                return true;
            }
        }
        return false;
    }
    resolveInterceptResult(result) {
        if (result instanceof Promise) {
            return result;
        }
        return Promise.resolve(result);
    }
    pathToRoute(path, props, options) {
        for (const data of this.routeDataSet.values()) {
            if (!data.regexp) {
                continue;
            }
            const [pathName, queryString] = path.split('?');
            if (data.regexp.exec(pathName)) {
                const moduleName = data.moduleName;
                const params = pathParams(pathName);
                const query = queryParams(queryString);
                props = { ...query, ...params, ...props };
                const dependencies = this.routeDependencies(data);
                const mode = data.mode;
                return { moduleName, props, dependencies, mode, options };
            }
        }
    }
    routeDependencies(routeData) {
        let dependencies = [];
        let data = routeData;
        while (data && data.dependency) {
            dependencies.push(data.dependency);
            data = this.routeDataSet.get(data.dependency);
        }
        return dependencies.reverse();
    }
    async processRoute(graph, target) {
        const handler = this.handlers.get(graph.layout);
        if (!handler) {
            return false;
        }
        const [consumed, childGraph] = await handler.process(graph, target);
        if (consumed && childGraph) {
            return await this.processRoute(childGraph, target);
        }
        return consumed;
    }
    async open(path, props = {}, options = {}) {
        this.inflateRouteData();
        const intercepted = await this.intercept(path);
        if (intercepted) {
            return;
        }
        const route = this.pathToRoute(path, props, options);
        if (!route) {
            return;
        }
        const graphArray = await Navigation.routeGraph();
        if (graphArray.length > 1) {
            for (let index = graphArray.length - 1; index > 0; index--) {
                const { mode: layoutMode } = graphArray[index];
                const { sceneId, moduleName } = await Navigation.currentRoute();
                if (layoutMode === 'present') {
                    await Navigation.dispatch(sceneId, 'dismiss', {
                        from: moduleName,
                    });
                }
                else if (layoutMode === 'modal') {
                    await Navigation.dispatch(sceneId, 'hideModal', {
                        from: moduleName,
                    });
                }
                else {
                    console.warn('尚未处理的 layout mode:' + layoutMode);
                }
            }
        }
        const routeGraph = graphArray[0];
        const consumed = await this.processRoute(routeGraph, route);
        if (consumed) {
            return;
        }
        const { sceneId, moduleName: from } = await Navigation.currentRoute();
        Navigation.dispatch(sceneId, 'closeMenu', { from });
        const { moduleName, mode: routeMode, props: initialProps } = route;
        let action = 'push';
        if (routeMode === 'present') {
            action = 'present';
        }
        else if (routeMode === 'modal') {
            action = 'showModal';
        }
        Navigation.dispatch(sceneId, action, {
            moduleName,
            props: initialProps,
            from,
            to: moduleName,
        });
    }
}
export const router = new Router();
export * from './drawer';
export * from './screen';
export * from './stack';
export * from './tabs';
